@@ -23,6 +23,10 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */
 #include "prefs.h"
 
 #include <the_Foundation/fileinfo.h>
+#include <assert.h>
+
+static_assert(offsetof(iPrefs, plainTextWrap) == offsetof(iPrefs, bools[plainTextWrap_PrefsBool]),
+              "memory layout mismatch (needs struct packing?)");
 
 void init_Prefs(iPrefs *d) {
     iForIndices(i, d->strings) {
@@ -40,8 +44,20 @@ void init_Prefs(iPrefs *d) {
     d->uiAnimations      = iTrue;
     d->uiScale           = 1.0f; /* default set elsewhere */
     d->zoomPercent       = 100;
+    d->navbarActions[0]  = back_ToolbarAction;
+    d->navbarActions[1]  = forward_ToolbarAction;
+    d->navbarActions[2]  = sidebar_ToolbarAction;
+    d->navbarActions[3]  = home_ToolbarAction;
+#if defined (iPlatformAndroidMobile)
+    /* Android has a system-wide back button so no need to have a duplicate. */
+    d->toolbarActions[0] = closeTab_ToolbarAction;
+#else
+    d->toolbarActions[0] = back_ToolbarAction;
+#endif
+    d->toolbarActions[1] = forward_ToolbarAction;
     d->sideIcon          = iTrue;
     d->hideToolbarOnScroll = iTrue;
+    d->blinkingCursor    = iTrue;
     d->pinSplit          = 1;
     d->time24h           = iTrue;
     d->returnKey         = default_ReturnKeyBehavior;
