@@ -111,6 +111,7 @@ struct Impl_TouchState {
     double momFrictionPerStep;
     double lastMomTime;
     iInt2 currentTouchPos; /* for emulating SDL_GetMouseState() */
+    iInt2 latestLongPressStartPos;
 };
 
 static iTouchState *touchState_(void) {
@@ -260,6 +261,11 @@ static iFloat3 gestureVector_Touch_(const iTouch *d) {
     return sub_F3(d->pos[0], d->pos[lastIndex]);
 }
 
+static uint32_t gestureSpan_Touch_(const iTouch *d) {
+    const size_t lastIndex = iMin(d->posCount - 1, lastIndex_Touch_);
+    return d->posTime[0] - d->posTime[lastIndex];
+}
+
 static void update_TouchState_(void *ptr) {
     iWindow *win = get_Window();
     const iWidget *oldHover = win->hover;
@@ -308,6 +314,7 @@ static void update_TouchState_(void *ptr) {
             }
             if (!touch->isTapAndHold && nowTime - touch->startTime >= longPressSpanMs_ &&
                 touch->affinity) {
+                touchState_()->latestLongPressStartPos = initF3_I2(touch->pos[0]);
                 dispatchClick_Touch_(touch, SDL_BUTTON_RIGHT);
                 touch->isTapAndHold = iTrue;
                 touch->hasMoved = iFalse;
@@ -593,9 +600,18 @@ iBool processEvent_Touch(const SDL_Event *ev) {
                     divvf_F3(&touch->accum, 6);
                     divfv_I2(&pixels, 6);
                     /* Allow scrolling a scrollable widget. */
-                    iWidget *flow = findOverflowScrollable_Widget(touch->affinity);
-                    if (flow) {
-                        touch->affinity = flow;
+                    if (touch->affinity && touch->affinity->flags2 & slidingSheetDraggable_WidgetFlag2) {
+                        extern iWidgetClass Class_SidebarWidget; /* The only type of sliding sheet for now. */
+                        iWidget *slider = findParentClass_Widget(touch->affinity, &Class_SidebarWidget);
+                        if (slider) {
+                            touch->affinity = slider;
+                        }
+                    }
+                    else {
+                        iWidget *flow = findOverflowScrollable_Widget(touch->affinity);
+                        if (flow) {
+                            touch->affinity = flow;
+                        }
                     }
                 }
                 else {
@@ -621,11 +637,13 @@ iBool processEvent_Touch(const SDL_Event *ev) {
             if (touch->axis == y_TouchAxis) {
                 pixels.x = 0;
             }
-//            printf("%p (%s) py: %i wy: %f acc: %f edge: %d\n",
-//                   touch->affinity,
-//                   class_Widget(touch->affinity)->name,
-//                   pixels.y, y_F3(amount), y_F3(touch->accum),
-//                   touch->edge);
+#if 0
+            printf("%p (%s) py: %i wy: %f acc: %f edge: %d\n",
+                   touch->affinity,
+                   class_Widget(touch->affinity)->name,
+                   pixels.y, y_F3(amount), y_F3(touch->accum),
+                   touch->edge);
+#endif
             if (pixels.x || pixels.y) {
                 //setFocus_Widget(NULL);
                 dispatchMotion_Touch_(touch->startPos /*pos[0]*/, 0);
@@ -661,11 +679,14 @@ iBool processEvent_Touch(const SDL_Event *ev) {
 #endif
             if (touch->edge && !isStationary_Touch_(touch)) {
                 const iFloat3 gesture = gestureVector_Touch_(touch);
+                const uint32_t duration = gestureSpan_Touch_(touch);
                 const float pixel = window->pixelRatio;
                 const int moveDir = x_F3(gesture) < -pixel ? -1 : x_F3(gesture) > pixel ? +1 : 0;
                 const int didAbort = (touch->edge == left_TouchEdge  && moveDir < 0) ||
                                      (touch->edge == right_TouchEdge && moveDir > 0);
-                postCommandf_App("edgeswipe.ended abort:%d side:%d id:%llu", didAbort, touch->edge, touch->id);
+                postCommandf_App("edgeswipe.ended abort:%d side:%d id:%llu speed:%d", didAbort,
+                                 touch->edge, touch->id,
+                                 (int) (duration > 0 ? length_F3(gesture) / (duration / 1000.0f) : 0));
                 remove_ArrayIterator(&i);
                 continue;
             }
@@ -689,8 +710,8 @@ iBool processEvent_Touch(const SDL_Event *ev) {
             }
             /* Edge swipes do not generate momentum. */
             const size_t lastIndex = iMin(touch->posCount - 1, lastIndex_Touch_);
-            const uint32_t duration = nowTime - touch->startTime;
             const iFloat3 gestureVector = sub_F3(pos, touch->pos[lastIndex]);
+            const uint32_t duration = nowTime - touch->startTime;
             iFloat3 velocity = zero_F3();
 #if 0
             if (touch->edge && fabsf(2 * x_F3(gestureVector)) > fabsf(y_F3(gestureVector)) &&
@@ -805,10 +826,24 @@ void widgetDestroyed_Touch(iWidget *widget) {
     }
 }
 
+void transferAffinity_Touch(iWidget *src, iWidget *dst) {
+    iTouchState *d = touchState_();
+    iForEach(Array, i, d->touches) {
+        iTouch *touch = i.value;
+        if (touch->affinity == src) {
+            touch->affinity = dst;
+        }
+    }
+}
+
 iInt2 latestPosition_Touch(void) {
     return touchState_()->currentTouchPos;
 }
 
+iInt2 latestTapPosition_Touch(void) {
+    return touchState_()->latestLongPressStartPos;
+}
+
 iBool isHovering_Touch(void) {
     iTouchState *d = touchState_();
     if (numFingers_Touch() == 1) {
